﻿using Microsoft.Extensions.Logging;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using VetClinicContracts.BindingModels;
using VetClinicContracts.BusinessLogicsContracts;
using VetClinicContracts.SearchModels;
using VetClinicContracts.StoragesContracts;
using VetClinicContracts.ViewModels;

namespace VetClinicBusinessLogic.BusinessLogics
{
	public class MedicineLogic : IMedicineLogic
	{
		private readonly ILogger _logger;
		private readonly IMedicineStorage _medicineStorage;
		public MedicineLogic(ILogger<MedicineLogic> logger, IMedicineStorage iceCreamStorage)
		{
			_logger = logger;
			_medicineStorage = iceCreamStorage;
		}

		public List<MedicineViewModel>? ReadList(MedicineSearchModel? model)
		{
			_logger.LogInformation("ReadList. MedicineName:{MedicineName}. Id:{ Id}", model?.MedicineName, model?.Id);
			var list = model == null ? _medicineStorage.GetFullList() :
_medicineStorage.GetFilteredList(model);
			if (list == null)
			{
				_logger.LogWarning("ReadList return null list");
				return null;
			}
			_logger.LogInformation("ReadList. Count:{Count}", list.Count);
			return list;
		}
		public MedicineViewModel? ReadElement(MedicineSearchModel model)
		{
			if (model == null)
			{
				throw new ArgumentNullException(nameof(model));
			}
			_logger.LogInformation("ReadElement. MedicineName:{MedicineName}.Id:{ Id}", model.MedicineName, model.Id);
			var element = _medicineStorage.GetElement(model);
			if (element == null)
			{
				_logger.LogWarning("ReadElement element not found");
				return null;
			}
			_logger.LogInformation("ReadElement find. Id:{Id}", element.Id);
			return element;
		}

		public bool Create(MedicineBindingModel model)
		{
			CheckModel(model);
			if (_medicineStorage.Insert(model) == null)
			{
				_logger.LogWarning("Insert operation failed");
				return false;
			}
			return true;
		}

		public bool Update(MedicineBindingModel model)
		{
			CheckModel(model);
			if (_medicineStorage.Update(model) == null)
			{
				_logger.LogWarning("Update operation failed");
				return false;
			}
			return true;
		}
		public bool Delete(MedicineBindingModel model)
		{
			CheckModel(model, false);
			_logger.LogInformation("Delete. Id:{Id}", model.Id);
			if (_medicineStorage.Delete(model) == null)
			{
				_logger.LogWarning("Delete operation failed");
				return false;
			}
			return true;
		}

		private void CheckModel(MedicineBindingModel model, bool withParams =
true)
		{
			if (model == null)
			{
				throw new ArgumentNullException(nameof(model));
			}
			if (!withParams)
			{
				return;
			}
			if (string.IsNullOrEmpty(model.MedicineName))
			{
				throw new ArgumentNullException("Нет названия медикамента",
			   nameof(model.MedicineName));
			}
			if (model.Price <= 0)
			{
				throw new ArgumentNullException("Цена медикамента должна быть больше 0", nameof(model.Price));
			}
			_logger.LogInformation("Medicine. Medicine:{MedicineName}. Price:{ Price }. Id: { Id}", model.MedicineName, model.Price, model.Id);
			var element = _medicineStorage.GetElement(new MedicineSearchModel
			{
				MedicineName = model.MedicineName
			});
			if (element != null && element.Id != model.Id)
			{
				throw new InvalidOperationException("Медикамент с таким названием уже есть");
			}
		}
	}
}
